#! /bin/sh /usr/share/dpatch/dpatch-run
## opt-42-x_MainMenuHooks.dpatch by Frank Schmirler <vdrdev@schmirler.de>
##
## All lines beginning with `## DP:' are a description of the patch.
## DP: This patch allows plugins to replace the VDR mainmenus "Schedule",
## DP: "Channels", "Timers" and "Recordings" by a different implementation.
## DP: * History
## DP: 2010-10-15: Version 1.0.1
## DP: - return a cOsdObject instead of its subclass cOsdMenu (thanks to
## DP:   Joe_D@vdrportal)
## DP: - version number defines in config.h now follow the ususal conventions:
## DP:   MAINMENUHOOKSVERSNUM is now a number, the newly added define
## DP:   MAINMENUHOOKSVERSION is a string (suggested by gnapheus@vdrportal)
## DP: - patch is now based on VDR 1.6.0
## DP: - updated documentation
## DP:
## DP: 2007-02-26: Version 1.0
## DP: - Initial revision.
## DP:
## DP: * Authors:
## DP: Tobias Grimm  <vdr at e-tobi dot net>
## DP: Martin Prochnow  <nordlicht at martins-kabuff dot de>
## DP: Frank Schmirler  <vdrdev at schmirler dot de>
## DP: Christian Wieninger  <cwieninger at gmx dot de>
## DP:
## DP: * Description:
## DP: This patch allows plugins to replace the VDR mainmenus "Schedule",
## DP: "Channels", "Timers" and "Recordings" by a different implementation.
## DP:
## DP: The patch is based on a suggestion of Christian Wieninger back in 2006
## DP: (http://www.linuxtv.org/pipermail/vdr/2006-March/008234.html). It is
## DP: meant to be an interim solution for VDR 1.4 until (maybe) VDR 1.5
## DP: introduces an official API for this purpose.
## DP:
## DP:
## DP: * Notes for plugin authors
## DP: The following code sample shows the required plugin code for replacing
## DP: the original Schedule menu:
## DP:
## DP: bool cMyPlugin::Service(const char *Id, void *Data)
## DP: {
## DP:   cOsdMenu **menu = (cOsdMenu**) Data;
## DP:   if (MySetup.replaceSchedule &&
## DP:             strcmp(Id, "MainMenuHooksPatch-v1.0::osSchedule") == 0) {
## DP:     if (menu)
## DP:       *menu = (cOsdMenu*) MainMenuAction();
## DP:     return true;
## DP:   }
## DP:   return false;
## DP: }
## DP:
## DP: Since patch version 1.0.1 the service call may return a cOsdObject
## DP: instead of a cOsdMenu. Use "#ifdef MAINMENUHOOKSVERSION" to detect
## DP: version 1.0.1.
## DP:
## DP: A plugin can replace more than one menu at a time. Simply replace the
## DP: call to MainMenuAction() in the sample above by appropriate code.
## DP:
## DP: Note that a plugin *should* offer a setup option which allows the user
## DP: to enable or disable the replacement. "Disabled" would be a reasonable
## DP: default setting. By testing for define MAINMENUHOOKSVERSNUM, a plugin
## DP: can leave the setup option out at compiletime.
## DP:
## DP: In case there is an internal problem when trying to open the replacement
## DP: menu, it is safe to return true even though Data is NULL. However an
## DP: OSD message should indicate the problem to the user.
## DP:
## DP: Feel free to ship this patch along with your plugin. However if you
## DP: think you need to modify the patch, we'd encourage you to contact the
## DP: authors first or at least use a service id which differs in more than
## DP: just the version number.

@DPATCH@
diff -urNad vdr-1.7.16/config.h vdr-1.7.16-mainmenuhooks/config.h
--- vdr-1.7.16/config.h	2010-11-18 12:21:40.230915881 +0100
+++ vdr-1.7.16-mainmenuhooks/config.h	2010-11-18 12:45:47.125416832 +0100
@@ -42,6 +42,10 @@
 
 #define REMOTEINSTANTVERSION 1.0
 
+// The MainMenuHook Patch's version number:
+#define MAINMENUHOOKSVERSION "1.0.1"
+#define MAINMENUHOOKSVERSNUM 10001  // Version * 10000 + Major * 100 + Minor
+
 #define MAXPRIORITY 99
 #define MAXLIFETIME 99
 
diff -urNad vdr-1.7.16/menu.c vdr-1.7.16-mainmenuhooks/menu.c
--- vdr-1.7.16/menu.c	2010-11-18 12:21:40.242905168 +0100
+++ vdr-1.7.16-mainmenuhooks/menu.c	2010-11-18 12:45:06.709414030 +0100
@@ -3732,15 +3732,31 @@
 
   // Initial submenus:
 
+  cOsdObject *menu = NULL;
   switch (State) {
-    case osSchedule:   AddSubMenu(new cMenuSchedule); break;
-    case osChannels:   AddSubMenu(new cMenuChannels); break;
-    case osTimers:     AddSubMenu(new cMenuTimers); break;
-    case osRecordings: AddSubMenu(new cMenuRecordings(NULL, 0, true)); break;
-    case osSetup:      AddSubMenu(new cMenuSetup); break;
-    case osCommands:   AddSubMenu(new cMenuCommands(tr("Commands"), &Commands)); break;
+    case osSchedule:
+        if (!cPluginManager::CallFirstService("MainMenuHooksPatch-v1.0::osSchedule", &menu))
+            menu = new cMenuSchedule;
+        break;
+    case osChannels:
+        if (!cPluginManager::CallFirstService("MainMenuHooksPatch-v1.0::osChannels", &menu))
+            menu = new cMenuChannels;
+        break;
+    case osTimers:
+        if (!cPluginManager::CallFirstService("MainMenuHooksPatch-v1.0::osTimers", &menu))
+            menu = new cMenuTimers;
+        break;
+    case osRecordings:
+        if (!cPluginManager::CallFirstService("MainMenuHooksPatch-v1.0::osRecordings", &menu))
+            menu = new cMenuRecordings(NULL, 0, true);
+        break;
+    case osSetup:      menu = new cMenuSetup; break;
+    case osCommands:   menu = new cMenuCommands(tr("Commands"), &Commands); break;
     default: break;
     }
+  if (menu)
+     if (menu->IsMenu())
+        AddSubMenu((cOsdMenu *) menu);
 }
 
 cOsdObject *cMenuMain::PluginOsdObject(void)
@@ -3900,18 +3916,40 @@
 
   cOsdMenu *menu = NULL;
   switch (state) {
-    case osSchedule:   return AddSubMenu(new cMenuSchedule);
-    case osChannels:   return AddSubMenu(new cMenuChannels);
-    case osTimers:     if (MenuOrgPatch::IsCustomMenuAvailable())
-                       {
-                            stopReplayItem = NULL;
-                            cancelEditingItem = NULL;
-                            stopRecordingItem = NULL;
-                       }
-                       return AddSubMenu(new cMenuTimers);
-    case osRecordings: return AddSubMenu(new cMenuRecordings);
-    case osSetup:      return AddSubMenu(new cMenuSetup);
-    case osCommands:   return AddSubMenu(new cMenuCommands(tr("Commands"), &Commands));
+    case osSchedule:
+        if (!cPluginManager::CallFirstService("MainMenuHooksPatch-v1.0::osSchedule", &menu))
+            menu = new cMenuSchedule;
+        else
+            state = osContinue;
+        break;
+    case osChannels:
+        if (!cPluginManager::CallFirstService("MainMenuHooksPatch-v1.0::osChannels", &menu))
+            menu = new cMenuChannels;
+        else
+            state = osContinue;
+        break;
+    case osTimers:
+        if (!cPluginManager::CallFirstService("MainMenuHooksPatch-v1.0::osTimers", &menu))
+        {
+            if (MenuOrgPatch::IsCustomMenuAvailable())
+            {
+                 stopReplayItem = NULL;
+                 cancelEditingItem = NULL;
+                 stopRecordingItem = NULL;
+            }
+            menu = new cMenuTimers;
+        }
+        else
+            state = osContinue;
+        break;
+    case osRecordings:
+        if (!cPluginManager::CallFirstService("MainMenuHooksPatch-v1.0::osRecordings", &menu))
+            menu = new cMenuRecordings;
+        else
+            state = osContinue;
+        break;
+    case osSetup:      menu = new cMenuSetup; break;
+    case osCommands:   menu = new cMenuCommands(tr("Commands"), &Commands); break; 
     case osStopRecord: if (Interface->Confirm(tr("Stop recording?"))) {
                           cOsdItem *item = Get(Current());
                           if (item) {
@@ -4000,6 +4038,12 @@
                default:      break;
                }
     }
+  if (menu) {
+     if (menu->IsMenu())
+        return AddSubMenu((cOsdMenu *) menu);
+     pluginOsdObject = menu;
+     return osPlugin;
+  }
   if (!HasSubMenu() && Update(HadSubMenu))
      Display();
   if (Key != kNone) {
